package com.example.aspdemo;

import android.content.Context;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Looper;
import android.util.JsonReader;
import android.util.Log;

import androidx.annotation.NonNull;

import com.aliyun.wuying.aspsdk.aspengine.IASPEngine;

import java.io.IOException;
import java.io.StringReader;
import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Random;
import java.util.TimeZone;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import okhttp3.OkHttpClient;

public class ADHelper {
    public static final Pattern ORGANIZATION_ID_PATTERN1 = Pattern.compile("(?<RegionId>cn[-][a-z]+)[+]dir[-](?<RegionNumber>[0-9]+)");
    public static final Pattern ORGANIZATION_ID_PATTERN2 = Pattern.compile("[a-z0-9]+[.][a-z0-9]+");

    private final static String TAG = "ADHelper";

    private final static String DEFAULT_REGION_ID = "cn-shanghai";

    private static Object sObjLocker = new Object();
    private static ADHelper sInstance = null;

    private String mRegionId = null;
    private String mRegionNumber = null;
    private String mOrganizationId = null;
    BigInteger mRandClientId = null;
    Context mContext = null;

    public interface IADLoginResponse {
        void onSuccess(String response);
        void onFailed(long errorCode);
    }

    public interface IADDesktopListResponse {
        void onSuccess(List<Desktop> desktopList);
        void onFailed(long errorCode);
    }

    public interface IADConnectionTicketResponse {
        void onSuccess(String ticket);
        void onFailed(long errorCode);
    }

    public class Desktop {
        public String mId;
        public String mName;
        public String mOSType;
        public String mStatus;
        public String mGroupId;
        public String mAppId;
        public String mProductType;
        public String mVersion;
        public String mConnectionProperties;
        public IASPEngine engine;
    }

    interface IADResponseHandler {
        void handleSuccessResponse(String response);
        void handleErrorResponse(int code, String message);
    }

    abstract class BaseHelper {
        protected String mLoginToken = null;
        protected String mSessionId = null;
        protected String mProductType;

        private HandlerThread mThread;
        private Handler mHandler;
        private Handler mMainHandler;

        BaseHelper() {
            mThread = new HandlerThread("ADHelper");
            mThread.start();
            Looper looper = mThread.getLooper();
            mHandler = new Handler(looper);
            mMainHandler = new Handler(Looper.getMainLooper());
        }

        void execute(String url, IADResponseHandler handler) {
            mHandler.post(new Runnable() {
                @Override
                public void run() {
                    Log.i(TAG, "Request url: " + url);
                    OkHttpClient client = new OkHttpClient.Builder()
                            .dns(new CustomDns())
                            .build();

                    okhttp3.Request request = new okhttp3.Request.Builder()
                            .url(url)
                            .get()
                            .build();

                    try (okhttp3.Response response = client.newCall(request).execute()) {
                        if (response.isSuccessful()) {
                            String responseBody = response.body().string();
                            Log.i(TAG, responseBody);
                            mMainHandler.post(new Runnable() {
                                @Override
                                public void run() {
                                    handler.handleSuccessResponse(responseBody);
                                }
                            });
                        } else {
                            Log.i(TAG, "error code: " + response.code());
                            Log.i(TAG, "error message: " + response.message());
                            Log.i(TAG, "error body: " + response.toString());
                            mMainHandler.post(new Runnable() {
                                @Override
                                public void run() {
                                    handler.handleErrorResponse(response.code(), response.message());
                                }
                            });
                        }
                    } catch (Exception ex) {

                    }
                }
            });

/*
            StringRequest stringRequest = new StringRequest(Request.Method.GET, url,
                    new Response.Listener<String>() {
                        @Override
                        public void onResponse(String response) {
                            Log.i(TAG, response);
                            handler.handleSuccessResponse(response);
                        }
                    }, new Response.ErrorListener() {
                @Override
                public void onErrorResponse(VolleyError error) {
                    if (error != null && error.networkResponse != null) {
                        Log.i(TAG, "error code: " + error.networkResponse.statusCode);
                        Log.i(TAG, "error response: " + new String(error.networkResponse.data));
                        handler.handleErrorResponse(error.networkResponse.statusCode, error.getLocalizedMessage());
                    } else {
                        Log.i(TAG, "error: " + error);
                        handler.handleErrorResponse(-1, null);
                    }
                }
            });*/

            // Add the request to the RequestQueue.
            //mQueue.add(stringRequest);
        }

        Uri.Builder getBaseADUri() {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
            sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
            String currentDateTimeString = sdf.format(new Date());
            return Uri.parse("https://" + mProductType + "." + mRegionId + ".aliyuncs.com/?")
                    .buildUpon()
                    .appendQueryParameter("ClientId", "Client" + mRandClientId)
                    .appendQueryParameter("ClientOS", "Android 10")
                    .appendQueryParameter("ClientVersion", "1.0.0")
                    .appendQueryParameter("Format", "json")
                    .appendQueryParameter("RegionId", mRegionId)
                    .appendQueryParameter("SignatureNonce",
                            Calendar.getInstance(TimeZone.getTimeZone("GMT")).getTimeInMillis() + "" + (long)Math.ceil(Math.random() * 10000))
                    .appendQueryParameter("Timestamp", currentDateTimeString);
        }

        abstract void getConnectionTicket(Desktop desktop, final IADConnectionTicketResponse adConnectionTicketResposne);
        abstract void getDesktopList(final IADDesktopListResponse adDesktopListResponse);
        abstract void login(String username, String password, final IADLoginResponse adLoginResponse);
        abstract Uri.Builder populateBaseADUri();
    }

    class AppStreamHelper extends BaseHelper {
        AppStreamHelper() {
            mProductType = "appstream-center";
        }

        void getConnectionTicket(Desktop desktop, final IADConnectionTicketResponse adConnectionTicketResposne) {
            getConnectionTicket(desktop, null, 0, adConnectionTicketResposne);
        }


        private void getConnectionTicket(Desktop desktop, final String taskId,
                                         final int currentLevel,
                                         final IADConnectionTicketResponse adConnectionTicketResposne) {
            String url = populateBaseADUri()
                    .appendQueryParameter("Action", "GetConnectionTicket")
                    .appendQueryParameter("LoginToken", mLoginToken)
                    .appendQueryParameter("ResourceId", desktop.mId)
                    .appendQueryParameter("AppInstanceGroupId", desktop.mGroupId)
                    .appendQueryParameter("SessionId", mSessionId)
                    .appendQueryParameter("AcsProduct", mProductType)
                    .appendQueryParameter("product", mProductType)
                    .appendQueryParameter("Version", "2021-09-03")
                    .appendQueryParameter("ProductType", desktop.mProductType)
                    .appendQueryParameter("AppVersion", desktop.mVersion)
                    .appendQueryParameter("AppId", desktop.mAppId)
                    .appendQueryParameter("Language", "zh_CN")
                    .appendQueryParameter("ConnectionProperties", desktop.mConnectionProperties)
                    .appendQueryParameter("TaskId", taskId == null ? "" : taskId)
                    .build().toString();

            execute(url, new IADResponseHandler() {
                @Override
                public void handleSuccessResponse(String response) {
                    JsonReader reader = new JsonReader(new StringReader(response));
                    boolean retry = false;
                    String nextTaskId = taskId;
                    int nextLevel = currentLevel + 1;
                    String taskStatus = null;
                    String connectionTicket = null;
                    try {
                        reader.beginObject();
                        while (reader.hasNext()) {
                            final String name = reader.nextName();
                            if (name.equals("TaskId")) {
                                nextTaskId = reader.nextString();
                            } else if (name.equals("TaskStatus")) {
                                taskStatus = reader.nextString();
                                retry = nextLevel < 10 && !taskStatus.equalsIgnoreCase("FINISHED");
                            } else if (name.equals("Ticket")) {
                                connectionTicket = reader.nextString();
                            } else {
                                reader.skipValue();
                            }
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }

                    if (retry) {
                        getConnectionTicket(desktop, nextTaskId, nextLevel, adConnectionTicketResposne);
                    } else if (taskStatus == null || !taskStatus.equalsIgnoreCase("FINISHED") || connectionTicket == null) {
                        Log.e(TAG, "Cannot get connection ticket after 10 retries");
                        adConnectionTicketResposne.onFailed(Long.MAX_VALUE);
                    } else {

                        adConnectionTicketResposne.onSuccess(connectionTicket);
                    }
                }

                @Override
                public void handleErrorResponse(int code, String message) {
                    adConnectionTicketResposne.onFailed(code);
                }
            });
        }

        void getDesktopList(final IADDesktopListResponse adDesktopListResponse) {
            String currentProductType = mProductType;
            mProductType = "ecd";
            String url = populateBaseADUri()
                    .appendQueryParameter("Action", "DescribeUserResources")
                    .appendQueryParameter("MaxResults", "500")
                    .appendQueryParameter("Product", mProductType)
                    .appendQueryParameter("ResourceTypes.1", "App")
                    .appendQueryParameter("SortType", "ASC")
                    .appendQueryParameter("AutoRefresh", "true")
                    //.appendQueryParameter("NextToken", "")
                    .appendQueryParameter("ProtocolType", "ASP")
                    .appendQueryParameter("QueryFotaUpdate", "true")
                    .appendQueryParameter("RefreshFotaUpdate", "true")
                    .appendQueryParameter("Language", "zh_CN")
                    .appendQueryParameter("ResourceTypes.2", "Desktop")
                    .appendQueryParameter("Scene", "desktop")
                    .appendQueryParameter("AccessType", "INTERNET")
                    .appendQueryParameter("OrderBy", "AssignTime")
                    .appendQueryParameter("ResourceTypes.4", "CloudDrive")
                    .appendQueryParameter("Version", "2020-10-02")
                    .appendQueryParameter("ResourceTypes.3", "DesktopGroup")
                    .appendQueryParameter("LoginToken", mLoginToken)
                    .appendQueryParameter("SessionId", mSessionId)
                    .build().toString();

            execute(url, new IADResponseHandler() {
                @Override
                public void handleSuccessResponse(String response) {
                    ArrayList<Desktop> desktops = new ArrayList<>();

                    JsonReader reader = new JsonReader(new StringReader(response));
                    try {
                        reader.beginObject();
                        while (reader.hasNext()) {
                            final String name = reader.nextName();
                            if (name.equals("Resources")) {
                                reader.beginArray();
                                while (reader.hasNext()) {
                                    Desktop desktop = new Desktop();
                                    reader.beginObject();
                                    while (reader.hasNext()) {
                                        final String n = reader.nextName();
                                        if (n.equals("ResourceName")) {
                                            desktop.mName = reader.nextString();
                                        } else if (n.equals("ResourceId")) {
                                            desktop.mId = reader.nextString();
                                        } else if (n.equals("ResourceStatus")) {
                                            desktop.mStatus = reader.nextString();
                                        } else if (n.equals("OsType")) {
                                            desktop.mOSType = reader.nextString();
                                        } else if (n.equals("ResourceGroupId")) {
                                            desktop.mGroupId = reader.nextString();
                                        } else if (n.equals("ProductType")) {
                                            desktop.mProductType = reader.nextString();
                                        } else if (n.equals("Version")) {
                                            desktop.mVersion = reader.nextString();
                                        } else if (n.equals("ConnectionProperties")) {
                                            desktop.mConnectionProperties = reader.nextString();
                                        } else if (n.equals("AppId")) {
                                            desktop.mAppId = reader.nextString();
                                        } else {
                                            reader.skipValue();
                                        }
                                    }
                                    desktops.add(desktop);
                                    reader.endObject();
                                }
                                reader.endArray();
                            } else {
                                reader.skipValue();
                            }
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }

                    adDesktopListResponse.onSuccess(desktops);
                }

                @Override
                public void handleErrorResponse(int code, String message) {
                    adDesktopListResponse.onFailed(code);
                }
            });
            mProductType = currentProductType;
        }

        void login(String username, String password, final IADLoginResponse adLoginResponse) {
            String url = populateBaseADUri()
                    .appendQueryParameter("Action", "GetLoginToken")
                    .appendQueryParameter("CurrentStage", "ADPassword")
                    .appendQueryParameter("EndUserId", username)
                    .appendQueryParameter("Password", password)
                    .appendQueryParameter("OfficeSiteId", mOrganizationId)
                    .appendQueryParameter("Version", "2021-02-20")
                    .appendQueryParameter("AcsProduct", mProductType)
                    .appendQueryParameter("product", mProductType)
                    .appendQueryParameter("IdpId", "idpId-simple-end-user-id")
                    .appendQueryParameter("ProductType", "CloudApp")
                    .appendQueryParameter("IsSdp", "false")
                    .appendQueryParameter("Language", "zh_CN")
                    .appendQueryParameter("NetworkType", "INTERNET")
                    .build().toString();

            execute(url, new IADResponseHandler() {
                @Override
                public void handleSuccessResponse(String response) {
                    JsonReader reader = new JsonReader(new StringReader(response));
                    try {
                        reader.beginObject();
                        while (reader.hasNext()) {
                            final String name = reader.nextName();
                            if (name.equals("LoginToken")) {
                                mLoginToken = reader.nextString();
                            } else if (name.equals("SessionId")) {
                                mSessionId = reader.nextString();
                            } else {
                                reader.skipValue();
                            }
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }

                    adLoginResponse.onSuccess(response);
                }

                @Override
                public void handleErrorResponse(int code, String message) {
                    adLoginResponse.onFailed(code);
                }
            });
        }

        Uri.Builder populateBaseADUri() {
            return getBaseADUri();
        }
    }

    class DesktopHelper extends BaseHelper {
        DesktopHelper() {
            mProductType = "ecd";
        }

        void getConnectionTicket(Desktop desktop, final IADConnectionTicketResponse adConnectionTicketResposne) {
            getConnectionTicket(desktop, null, 0, adConnectionTicketResposne);
        }

        private void getConnectionTicket(Desktop desktop, final String taskId,
                                         final int currentLevel,
                                         final IADConnectionTicketResponse adConnectionTicketResposne) {
            String url = populateBaseADUri()
                    .appendQueryParameter("Action", "GetConnectionTicket")
                    .appendQueryParameter("LoginToken", mLoginToken)
                    .appendQueryParameter("DesktopId", desktop.mId)
                    .appendQueryParameter("SessionId", mSessionId)
                    .appendQueryParameter("TaskId", taskId == null ? "" : taskId)
                    .build().toString();

            execute(url, new IADResponseHandler() {
                @Override
                public void handleSuccessResponse(String response) {
                    JsonReader reader = new JsonReader(new StringReader(response));
                    boolean retry = false;
                    String nextTaskId = taskId;
                    int nextLevel = currentLevel + 1;
                    String taskStatus = null;
                    String connectionTicket = null;
                    try {
                        reader.beginObject();
                        while (reader.hasNext()) {
                            final String name = reader.nextName();
                            if (name.equals("TaskId")) {
                                nextTaskId = reader.nextString();
                            } else if (name.equals("TaskStatus")) {
                                taskStatus = reader.nextString();
                                retry = nextLevel < 10 && !taskStatus.equalsIgnoreCase("FINISHED");
                            } else if (name.equals("Ticket")) {
                                connectionTicket = reader.nextString();
                            } else {
                                reader.skipValue();
                            }
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }

                    if (retry) {
                        getConnectionTicket(desktop, nextTaskId, nextLevel, adConnectionTicketResposne);
                    } else if (taskStatus == null || !taskStatus.equalsIgnoreCase("FINISHED") || connectionTicket == null) {
                        Log.e(TAG, "Cannot get connection ticket after 10 retries");
                        adConnectionTicketResposne.onFailed(Long.MAX_VALUE);
                    } else {
                        adConnectionTicketResposne.onSuccess(connectionTicket);
                    }
                }

                @Override
                public void handleErrorResponse(int code, String message) {
                    adConnectionTicketResposne.onFailed(code);
                }
            });
        }

        void getDesktopList(final IADDesktopListResponse adDesktopListResponse) {
            String url = populateBaseADUri()
                    .appendQueryParameter("Action", "DescribeDesktops")
                    .appendQueryParameter("LoginToken", mLoginToken)
                    .appendQueryParameter("MaxResults", "100")
                    .appendQueryParameter("NextToken", "")
                    .appendQueryParameter("ProtocolType", "both")
                    .appendQueryParameter("SessionId", mSessionId)
                    .build().toString();

            execute(url, new IADResponseHandler() {
                @Override
                public void handleSuccessResponse(String response) {
                    ArrayList<Desktop> desktops = new ArrayList<>();
                    JsonReader reader = new JsonReader(new StringReader(response));
                    try {
                        reader.beginObject();
                        while (reader.hasNext()) {
                            final String name = reader.nextName();
                            if (name.equals("Desktops")) {
                                reader.beginArray();
                                while (reader.hasNext()) {
                                    Desktop desktop = new Desktop();
                                    reader.beginObject();
                                    while (reader.hasNext()) {
                                        final String n = reader.nextName();
                                        if (n.equals("DesktopName")) {
                                            desktop.mName = reader.nextString();
                                        } else if (n.equals("DesktopId")) {
                                            desktop.mId = reader.nextString();
                                        } else if (n.equals("DesktopStatus")) {
                                            desktop.mStatus = reader.nextString();
                                        } else if (n.equals("OsType")) {
                                            desktop.mOSType = reader.nextString();
                                        } else {
                                            reader.skipValue();
                                        }
                                    }
                                    desktops.add(desktop);
                                    reader.endObject();
                                }
                                reader.endArray();
                            } else {
                                reader.skipValue();
                            }
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }

                    adDesktopListResponse.onSuccess(desktops);
                }

                @Override
                public void handleErrorResponse(int code, String message) {
                    adDesktopListResponse.onFailed(code);
                }
            });
        }

        void login(String username, String password, final IADLoginResponse adLoginResponse) {
            String url = populateBaseADUri()
                    .appendQueryParameter("Action", "GetLoginToken")
                    .appendQueryParameter("CurrentStage", "ADPassword")
                    .appendQueryParameter("EndUserId", username)
                    .appendQueryParameter("Password", password)
                    .build().toString();

            execute(url, new IADResponseHandler() {
                @Override
                public void handleSuccessResponse(String response) {
                    JsonReader reader = new JsonReader(new StringReader(response));
                    try {
                        reader.beginObject();
                        while (reader.hasNext()) {
                            final String name = reader.nextName();
                            if (name.equals("LoginToken")) {
                                mLoginToken = reader.nextString();
                            } else if (name.equals("SessionId")) {
                                mSessionId = reader.nextString();
                            } else {
                                reader.skipValue();
                            }
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }

                    adLoginResponse.onSuccess(response);
                }

                @Override
                public void handleErrorResponse(int code, String message) {
                    adLoginResponse.onFailed(code);
                }
            });
        }

        Uri.Builder populateBaseADUri() {
            return getBaseADUri()
                    .appendQueryParameter("DirectoryId", mOrganizationId)
                    .appendQueryParameter("Version", "2020-10-02")
                    .appendQueryParameter("product", "ecs_ad");
        }
    }

    private BaseHelper mHelper;

    private ADHelper() {
        mRandClientId = new BigInteger(32, new Random());
    }

    static public ADHelper getInstance() {
        synchronized (sObjLocker) {
            if (sInstance == null) {
                sInstance = new ADHelper();
            }
        }

        return sInstance;
    }

    public void setContext(Context context) {
        mContext = context;
    }

    public void setRegionId(String regionId) {
        mRegionId = regionId;
    }

    public void setRegionNumber(String regionNumber) {
        mRegionNumber = regionNumber;
    }

    public void setOrganizationId(String organizationId) {
        mOrganizationId = organizationId;
        Matcher m = ORGANIZATION_ID_PATTERN1.matcher(mOrganizationId);
        if (m.matches()) {
            setRegionId(m.group("RegionId").toString());
            setRegionNumber(m.group("RegionNumber").toString());
            mHelper = new DesktopHelper();
        } else {
            setRegionId(DEFAULT_REGION_ID);
            mHelper = new AppStreamHelper();
        }
    }

    public void getConnectionTicket(Desktop desktop, final IADConnectionTicketResponse adConnectionTicketResposne) {
        //mHelper.getConnectionTicket(context, desktop, null, 0, adConnectionTicketResposne);
        mHelper.getConnectionTicket(desktop, adConnectionTicketResposne);
    }

    public void getDesktopList(final IADDesktopListResponse adDesktopListResponse) {
        if (mHelper != null) {
            mHelper.getDesktopList(adDesktopListResponse);
        }
    }

    public void login(String username, String password, final IADLoginResponse adLoginResponse) {
        mHelper.login(username, password, adLoginResponse);
    }
}
