package com.example.aspdemo;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.util.DisplayMetrics;
import android.util.Log;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import com.aliyun.wuying.aspsdk.aspengine.AspClientType;
import com.aliyun.wuying.aspsdk.aspengine.ConnectionConfig;
import com.aliyun.wuying.aspsdk.aspengine.IASPEngine;
import com.aliyun.wuying.aspsdk.aspengine.IThumbnailListener;
import com.aliyun.wuying.aspsdk.aspengine.ui.StreamView;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;

public class DesktopListActivity extends AppCompatActivity {
    private static final String TAG = "DesktopListActivity";

    private ProgressBar mLoadingProgressBar;
    private ListView mDesktopListView;
    private List<ADHelper.Desktop> mDesktopList = new ArrayList<>();
    private RecyclerView recyclerView;
    private OnItemClickListener itemClickListener;
    private RecyclerView.OnScrollListener scrollListener;

    class ListAdapter extends ArrayAdapter<ADHelper.Desktop> {

        public ListAdapter(Context context, int resource, List<ADHelper.Desktop> items) {
            super(context, resource, items);
        }

        @Override
        public View getView(int position, View view, ViewGroup parent) {
            View v = view;
            if (v == null) {
                v = LayoutInflater.from(DesktopListActivity.this).inflate(R.layout.desktop_list_item, null);
            }

            ADHelper.Desktop item = getItem(position);
            ImageView imageView = v.findViewById(R.id.desktop_image);
            if (item.mOSType != null && item.mOSType.equals("Windows")) {
                imageView.setImageResource(R.drawable.windows);
            }

            if("AndroidCloud".equals(item.mProductType)) {
                if(item.engine != null) {
                    item.engine.resumeThumbnailDisplay();
                }else {
                    ADHelper.getInstance().getConnectionTicket(item, new ADHelper.IADConnectionTicketResponse() {
                        @Override
                        public void onSuccess(String ticket) {
                            Log.e(TAG, "ticket = " + ticket);
                            IASPEngine engine = new IASPEngine.Builder()
                                    .enableRTC(false)
                                    .enableVDAgentCheck(true)
                                    .enableThumbnail(true)
                                    .build();
                            engine.registerThumbnailListener(new IThumbnailListener() {
                                @Override
                                public void onThumbnailUpdate(byte[] rgba, int width, int height) {
                                    Log.i(TAG, "onThumbnailUpdate width: " + width + ", height: " + height);
                                    Bitmap bitmap = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888);
                                    bitmap.copyPixelsFromBuffer(ByteBuffer.wrap(rgba));

                                    Handler handler = new Handler(Looper.getMainLooper());
                                    handler.post(new Runnable() {
                                        @Override
                                        public void run() {
                                            imageView.setImageBitmap(bitmap);
                                        }
                                    });
                                }
                            });
                            engine.setThumbnailDisplay(360, 200, 1);

                            ConnectionConfig cc = new ConnectionConfig();
                            cc.id = item.mId;
                            cc.connectionTicket = ticket;
                            cc.caFilePath = CAFileHelper.getCAFilePath(DesktopListActivity.this);
                            cc.useVPC = false;
                            cc.type = AspClientType.ASP_ENGINE_CLIENT_DESKTOP;
                            cc.user = "";
                            cc.uuid = "";
                            engine.start(cc);
                            item.engine = engine;
                        }

                        @Override
                        public void onFailed(long errorCode) {
                            Log.i(TAG, "Failed to get ticket from remote server!");
                        }
                    });
                }
            }

            TextView nameView = v.findViewById(R.id.desktop_name);
            TextView versionView = v.findViewById(R.id.desktop_status);

            nameView.setText(item.mName);
            versionView.setText(item.mStatus);

            return v;
        }
    };

    class DesktopRecycleViewAdapter extends RecyclerView.Adapter<DesktopRecycleViewAdapter.ViewHolder> {
        private List<ADHelper.Desktop> list;
        private OnItemClickListener listener;

        public DesktopRecycleViewAdapter(List<ADHelper.Desktop> list, OnItemClickListener listener) {
            this.list = list;
            this.listener = listener;
        }

        @NonNull
        @Override
        public DesktopRecycleViewAdapter.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.desktop_list_item, parent, false);
            return new ViewHolder(view);
        }

        @Override
        public void onBindViewHolder(@NonNull DesktopRecycleViewAdapter.ViewHolder holder, int position) {
            ADHelper.Desktop item = list.get(position);
            if (item.mOSType != null && item.mOSType.equals("Windows")) {
                holder.imageView.setImageResource(R.drawable.windows);
            }
            holder.nameView.setText(item.mName);
            holder.versionView.setText(item.mStatus);

            if("AndroidCloud".equals(item.mProductType)) {
                if(item.engine != null) {
                    item.engine.resumeThumbnailDisplay();
                }else {
                    ADHelper.getInstance().getConnectionTicket(item, new ADHelper.IADConnectionTicketResponse() {
                        @Override
                        public void onSuccess(String ticket) {
                            Log.e(TAG, " ticket = " + ticket);
                            IASPEngine engine = new IASPEngine.Builder()
                                    .enableRTC(false)
                                    .enableVDAgentCheck(true)
                                    .enableThumbnail(true)
                                    .build();
                            engine.registerThumbnailListener(new IThumbnailListener() {
                                @Override
                                public void onThumbnailUpdate(byte[] rgba, int width, int height) {
                                    Log.i(TAG, "onThumbnailUpdate width: " + width + ", height: " + height);
                                    Bitmap bitmap = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888);
                                    bitmap.copyPixelsFromBuffer(ByteBuffer.wrap(rgba));

                                    Handler handler = new Handler(Looper.getMainLooper());
                                    handler.post(new Runnable() {
                                        @Override
                                        public void run() {
                                            holder.imageView.setImageBitmap(bitmap);
                                        }
                                    });
                                }
                            });

                            engine.setThumbnailDisplay(360, 200, 1);

                            ConnectionConfig cc = new ConnectionConfig();
                            cc.id = item.mId;
                            cc.connectionTicket = ticket;
                            cc.caFilePath = CAFileHelper.getCAFilePath(DesktopListActivity.this);
                            cc.useVPC = false;
                            cc.type = AspClientType.ASP_ENGINE_CLIENT_DESKTOP;
                            cc.user = "";
                            cc.uuid = "";
                            engine.start(cc);
                            item.engine = engine;
                        }

                        @Override
                        public void onFailed(long errorCode) {
                            Log.i(TAG, "Failed to get ticket from remote server!");
                        }
                    });
                }
            }
        }

        @Override
        public int getItemCount() {
            return list.size();
        }
        class ViewHolder extends RecyclerView.ViewHolder {
            ImageView imageView;
            TextView nameView;
            TextView versionView;

            ViewHolder(View itemView) {
                super(itemView);
                imageView = itemView.findViewById(R.id.desktop_image);
                nameView = itemView.findViewById(R.id.desktop_name);
                versionView = itemView.findViewById(R.id.desktop_status);

                itemView.setOnClickListener(v -> {
                    if (listener != null) {
                        int position = getAdapterPosition();
                        if (position != RecyclerView.NO_POSITION) {
                            listener.onItemClick(position);
                        }
                    }
                });
            }
        }
    }

    public interface OnItemClickListener {
        void onItemClick(int position);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_desktop_list);

        Bundle b = getIntent().getExtras();
        String user = b.getString("user");
        setTitle(user);

        mLoadingProgressBar = findViewById(R.id.loading);
//        mDesktopListView = findViewById(R.id.desktop_list_view);

        recyclerView = findViewById(R.id.rv_desktop);

        DisplayMetrics displayMetrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(displayMetrics);
        int screenWidth = displayMetrics.widthPixels;

        int desiredItemWidthInDp = 200;
        int desiredItemWidth = (int) TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, desiredItemWidthInDp, getResources().getDisplayMetrics());

        // 计算列数
        int numberOfColumns = Math.max(1, screenWidth / desiredItemWidth);

        GridLayoutManager gridLayoutManager = new GridLayoutManager(this, numberOfColumns);
        recyclerView.setLayoutManager(gridLayoutManager);

        itemClickListener = position -> {
            mLoadingProgressBar.setVisibility(View.VISIBLE);
            ADHelper.Desktop desktop = mDesktopList.get(position);
            if(desktop.engine != null) {
                desktop.engine.suspendThumbnailDisplay();
            }
            final String desktopId = desktop.mId;
            Log.i(TAG, "open desktop: " + desktopId);
            ADHelper.getInstance().getConnectionTicket(desktop, new ADHelper.IADConnectionTicketResponse() {
                @Override
                public void onSuccess(String ticket) {
                    mLoadingProgressBar.setVisibility(View.GONE);
                    Bundle bundle = new Bundle();
                    bundle.putString(StreamView.CONFIG_DESKTOP_ID, desktopId);
                    bundle.putString(StreamView.CONFIG_CONNECTION_TICKET, ticket);
                    bundle.putString(StreamView.CONFIG_CA_FILE_PATH, CAFileHelper.getCAFilePath(DesktopListActivity.this));
                    //bundle.putBoolean(StreamView.CONFIG_PREFER_RTC_TRANSPORT, !desktop.mOSType.equalsIgnoreCase("android"));
                    bundle.putBoolean(StreamView.CONFIG_PREFER_RTC_TRANSPORT, true);
                    bundle.putBoolean(StreamView.CONFIG_ENABLE_VDAGENT_CHECK, true);
                    bundle.putBoolean(StreamView.CONFIG_ENABLE_STATISTICS, true);
                    bundle.putBoolean(StreamView.CONFIG_ENALBE_SCREEN_ORIENTATION_CHANGE, true);
                    bundle.putString("OSType", desktop.mOSType);
                    bundle.putInt(StreamView.CONFIG_CLIENT_TYPE, "CloudApp".equals(desktop.mProductType) ? 1 : 0);
                    Log.i(TAG, "open desktop type: " + desktop.mProductType);
                    //Intent intent = new Intent(
                    //        DesktopListActivity.this, CustomizedPlayerDemoActivity.class);
                    Intent intent = new Intent(
                            DesktopListActivity.this, StreamViewDemoActivity.class);
                    intent.putExtras(bundle);
                    intent.setFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
                    startActivity(intent);
                }

                @Override
                public void onFailed(long errorCode) {
                    mLoadingProgressBar.setVisibility(View.GONE);
                    Log.i(TAG, "Failed to get ticket from remote server!");
                    Toast.makeText(DesktopListActivity.this,
                            "Failed to get ticket from remote server!", Toast.LENGTH_LONG).show();
                }
            });
        };
        scrollListener = new RecyclerView.OnScrollListener() {
            int totalItemCount = 0;
            int firstVisibleItemPosition = 0;
            int lastVisibleItemPosition = 0;
            int oldLastVisibleItem = 0;
            int oldFirstVisibleItem = 0;

            @Override
            public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
                super.onScrolled(recyclerView, dx, dy);
                firstVisibleItemPosition = gridLayoutManager.findFirstVisibleItemPosition();
                lastVisibleItemPosition = gridLayoutManager.findLastVisibleItemPosition();
            }

            @Override
            public void onScrollStateChanged(@NonNull RecyclerView recyclerView, int newState) {
                super.onScrollStateChanged(recyclerView, newState);
                if (RecyclerView.SCROLL_STATE_IDLE == newState) {
                    if (oldLastVisibleItem != lastVisibleItemPosition ) {
                        totalItemCount = gridLayoutManager.getItemCount();
                        oldLastVisibleItem = lastVisibleItemPosition;
                        oldFirstVisibleItem = firstVisibleItemPosition;
                        Log.i(TAG, "firstVisibleItem: " + firstVisibleItemPosition + " lastVisibleItem: " + lastVisibleItemPosition);
                        for (int i = 0; i < totalItemCount; i++) {
                            ADHelper.Desktop desktop = mDesktopList.get(i);
                            if(desktop != null && desktop.engine != null) {
                                if (i >= oldFirstVisibleItem && i <= oldLastVisibleItem) {
                                    Log.i(TAG, "visible item: " + i);
                                    desktop.engine.resumeThumbnailDisplay();
                                } else {
                                    Log.i(TAG, "not visible item: " + i);
                                    desktop.engine.suspendThumbnailDisplay();
                                }
                            }
                        }
                    }
                }
            }
        };
        recyclerView.addOnScrollListener(scrollListener);
    }

    @Override
    protected void onResume() {
        super.onResume();

        mLoadingProgressBar.setVisibility(View.VISIBLE);

        ADHelper.getInstance().getDesktopList(new ADHelper.IADDesktopListResponse() {
            @Override
            public void onSuccess(List<ADHelper.Desktop> desktopList) {
                mLoadingProgressBar.setVisibility(View.GONE);
                Log.i(TAG, "desktop size " + desktopList.size());
                mDesktopList.clear();
                mDesktopList.addAll(desktopList);
//                mDesktopListView.setAdapter(new ListAdapter(DesktopListActivity.this, R.layout.desktop_list_item, desktopList));
                recyclerView.setAdapter(new DesktopRecycleViewAdapter(mDesktopList, itemClickListener));
            }

            @Override
            public void onFailed(long errorCode) {
                Toast.makeText(DesktopListActivity.this, "Error: " + errorCode, Toast.LENGTH_LONG);
            }
        });
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if(recyclerView != null) {
            recyclerView.removeOnScrollListener(scrollListener);
        }
    }
}