package com.example.aspdemo;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.media.MediaCodecInfo;
import android.media.MediaCodecList;
import android.media.MediaFormat;
import android.net.Uri;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.preference.PreferenceManager;
import android.provider.MediaStore;
import android.util.Log;
import android.util.Range;
import android.view.KeyEvent;
import android.view.Window;
import android.view.WindowManager;
import android.widget.TextView;

import android.widget.Toast;
import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

//import com.aliyun.wuying.aspsdk.aspengine.ClipboardData;
import com.aliyun.wuying.aspsdk.aspengine.ClipboardData;
import com.aliyun.wuying.aspsdk.aspengine.DataChannel;
import com.aliyun.wuying.aspsdk.aspengine.DataChannelConnectState;
import com.aliyun.wuying.aspsdk.aspengine.EDSAgentChannel;
import com.aliyun.wuying.aspsdk.aspengine.IASPEngineListener;
import com.aliyun.wuying.aspsdk.aspengine.IFileTransferListener;
import com.aliyun.wuying.aspsdk.aspengine.IOrientationUpdateListener;
import com.aliyun.wuying.aspsdk.aspengine.IRuntimeStatusInspector;
import com.aliyun.wuying.aspsdk.aspengine.IStatisticsListener;
import com.aliyun.wuying.aspsdk.aspengine.LyncChannel;
import com.aliyun.wuying.aspsdk.aspengine.ScreenRotation;
import com.aliyun.wuying.aspsdk.aspengine.ScreenRotationSource;
import com.aliyun.wuying.aspsdk.aspengine.XferErrorCode;
import com.aliyun.wuying.aspsdk.aspengine.ui.StreamView;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.StringWriter;
import java.util.ArrayList;

public class StreamViewDemoActivity extends Activity {
    private static final String TAG = "StreamViewDemoActivity";
    private static final int WRITE_REQUEST_CODE = 1;
    private StreamView mStreamView = null;
    private MenuBar mMenuBar = null;
    private Bundle mConfigs = null;
    private boolean mDumpData = false;
    private String mOSType = null;
    private TextView mStatisticInfoView;

    private IASPEngineListener mListener = new IASPEngineListener() {
        @Override
        public void onConnectionSuccess(int connectionId) {
            Log.i(TAG, "onConnectionSuccess");
            DialogHelper.getInstance().setState(DialogHelper.State.STATE_CONNECTED);

            //mStreamView.setVideoProfile(1280, 800, 30, null);
        }

        @Override
        public void onConnectionFailure(int errorCode, String errorMsg) {
            Log.i(TAG, "onConnectionFailure errorCode:" + errorCode + " errorMsg:" + errorMsg);
            DialogHelper.getInstance().setState(DialogHelper.State.STATE_CONNECTION_ERROR,
                    String.valueOf(errorCode), errorMsg);
        }

        @Override
        public void onEngineError(int errorCode, String errorMsg) {
            Log.i(TAG, "onEngineError errorCode:" + errorCode + " errorMsg:" + errorMsg);
            DialogHelper.getInstance().setState(DialogHelper.State.STATE_ENGINE_ERROR,
                    String.valueOf(errorCode), errorMsg);
        }

        @Override
        public void onDisconnected(int reason) {
            Log.i(TAG, "onDisconnected reason=" + reason);
            DialogHelper.getInstance().setState(DialogHelper.State.STATE_DISCONNECTED);
        }

        @Override
        public void onReconnect(int errorCode) {
            Log.i(TAG, "onReconnect");
            DialogHelper.getInstance().setState(DialogHelper.State.STATE_CONNECTING);
        }

        @Override
        public void onFirstFrameRendered(long timeCostMS) {
            Log.i(TAG, "onFirstFrameRendered");
        }

        @Override
        public void onPolicyUpdate(String policy) {
            Log.i(TAG, "policy=" + policy);
        }

        @Override
        public void onSessionSuccess() {

        }
    };

    @Override
    public void onConfigurationChanged(@NonNull Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
    }

    private IFileTransferListener mFileTransferListener = new IFileTransferListener() {
        @Override
        public void onServerSupportOSSFileTransfer() {
            Log.i(TAG, "onServerSupportOSSFileTransfer");
        }

        @Override
        public void onProgress(int taskId, int progress) {
            Log.i(TAG, "onProgress taskId=" + taskId + " progress=" + progress);
        }

        @Override
        public void onComplete(int taskId) {
            Log.i(TAG, "onComplete taskId=" + taskId);
        }

        @Override
        public void onError(int taskId, XferErrorCode errorCode) {
            Log.i(TAG, "onError taskId=" + taskId + " errorCode=" + errorCode);
        }
    };

    private WifiManager.WifiLock mWifiLowLatencyLock = null;
    private WifiManager.WifiLock mWifiHighPerfLock = null;

    private static final int MY_PERMISSIONS_REQUEST_READ_EXTERNAL_STORAGE = 100;
    private static final int MY_PERMISSIONS_REQUEST_CAMERA = 101;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        if (ContextCompat.checkSelfPermission(this, Manifest.permission.READ_EXTERNAL_STORAGE)
                != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(this,
                    new String[]{Manifest.permission.READ_EXTERNAL_STORAGE},
                    MY_PERMISSIONS_REQUEST_READ_EXTERNAL_STORAGE);
        }

        if (ContextCompat.checkSelfPermission(this, Manifest.permission.CAMERA)
                != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(this,
                    new String[]{Manifest.permission.CAMERA},
                    MY_PERMISSIONS_REQUEST_CAMERA);
        }

        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);

        setContentView(R.layout.activity_stream_view_demo);

        DialogHelper.getInstance().initialize(this);

        PreferenceManager.setDefaultValues(this, R.xml.root_preferences, false);
        SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);

        mConfigs = getIntent().getExtras();
        mOSType = mConfigs.getString("OSType");

        Log.i(TAG, "OSType is " + mOSType);

        mMenuBar = findViewById(R.id.menu_bar);
        mMenuBar.setActivity(this);

        mStatisticInfoView = findViewById(R.id.statistic_info);

        mStreamView = findViewById(R.id.stream_view);
        mStreamView.enableDesktopMode(false);
        mStreamView.scaleStreamVideo(pref.getBoolean("fit_video_content", true) ?
                StreamView.ScaleType.FIT_STREAM_CONTENT : StreamView.ScaleType.FILL_STREAM_VIEW);
        mStreamView.getASPEngineDelegate().setAlignStreamResolutionWithSurfaceSize(false);

        if (mDumpData) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                String[] permissions = {
                        Manifest.permission.READ_EXTERNAL_STORAGE,
                        Manifest.permission.WRITE_EXTERNAL_STORAGE
                };
                requestPermissions(permissions, WRITE_REQUEST_CODE);
            }
        }

        //mStreamView.getASPEngineDelegate().setVideoProfile(1798, 984,30, null);
        //mStreamView.getASPEngineDelegate().registerASPEngineListener(mListener);
        mStreamView.getASPEngineDelegate().registerStatisticsListener(new IStatisticsListener() {
            @Override
            public void onStatisticsInfoUpdate(StatisticsInfo statisticsInfo) {
                String infoToDisplay = String.valueOf(statisticsInfo.mNetworkLatencyMS);
                infoToDisplay += "/";
                infoToDisplay += String.format("%.3f", statisticsInfo.mDownstreamBandwithMBPerSecond);
                infoToDisplay += "/";
                infoToDisplay += String.format("%.3f", statisticsInfo.mUpstreamBandwithMBPerSecond);

                final String infoStringToDisplay = infoToDisplay;
                mStreamView.post(new Runnable() {
                    @Override
                    public void run() {
                        mStatisticInfoView.setText(infoStringToDisplay);
                    }
                });
            }
        });
        mStreamView.getASPEngineDelegate().registerASPEngineListener(mListener);
        if (mOSType.equalsIgnoreCase("android")) {
            mStreamView.getASPEngineDelegate().setAlignStreamResolutionWithSurfaceSize(false);
        }

        mStreamView.getASPEngineDelegate().registerOrientationUpdateListener(
            new IOrientationUpdateListener() {
                @Override
                public void onOrientationUpdate(ScreenRotationSource source, ScreenRotation orientation) {
                    Log.i(TAG, "onOrientationUpdate = " + orientation);
                }
            });

        mStreamView.getASPEngineDelegate().registerFileTransferListener(mFileTransferListener);
        mStreamView.setVideoProfile(1920, 1080, -1, null);


        mMenuBar.setOSType(mOSType);
        mMenuBar.setStreamView(mStreamView);

        mStreamView.getASPEngineDelegate().setGuestOSType(mOSType);

        mStreamView.getASPEngineDelegate().addDataChannel(new DataChannel("test1") {
            @Override
            public void onConnectStateChanged(DataChannelConnectState state) {
                Log.i(TAG, "Test1 dc connection state changed to " + state);
                if (state == DataChannelConnectState.OPEN) {
                    send("Test 1 is opened!".getBytes());
                }
            }

            @Override
            public void onReceiveData(byte[] buf) {
                Log.i(TAG, "Test1 dc received " + buf.length + " bytes data");
            }
        });

        mStreamView.getASPEngineDelegate().addDataChannel(new EDSAgentChannel() {
            @Override
            public void onReceiveData(byte[] buf) {
                Log.i(TAG, "EDSAgent channel received " + buf.length + " bytes data");
                send("Just a test".getBytes());
            }

            @Override
            public void onGEMSystemOptimizeProgressUpdate(byte[] buf) {
                Log.i(TAG, "EDSAgent channel received " + buf.length + " bytes data for gem progress update");
            }
        });

        mStreamView.getASPEngineDelegate().addLyncChannel(new LyncChannel("test2") {
            @Override
            protected void onConnectStateChanged(boolean connected) {
                Log.i(TAG, "Test2 dc connection state changed to " + connected);
                if (connected) {
                    sendString("Test 2 is opened!");
                }
            }

            @Override
            protected void onReceiveStringData(String val) {
                Log.i(TAG, "Test2 received string data " + val);
            }

            @Override
            protected void onReceiveRawData(byte[] val) {
                Log.i(TAG, "Test2 received raw data, size " + val.length);
            }
        });

        mStreamView.start(mConfigs);

        DialogHelper.getInstance().setState(DialogHelper.State.STATE_CONNECTING);
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        switch (requestCode) {
            case WRITE_REQUEST_CODE:
                for (int i = 0; i < permissions.length; i++) {
                    Log.i(TAG, "permission " + permissions[i] + " granted: "
                            + (grantResults[i] == PackageManager.PERMISSION_GRANTED));
                }
                break;
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == MenuBar.PICK_IMAGE_REQUEST && resultCode == RESULT_OK && data != null) {
            Uri selectedImage = data.getData();
            Log.i(TAG, "Select image " + selectedImage);
            //mStreamView.copyImageUri(selectedImage);
        } else if (requestCode == MenuBar.OPEN_FILE_PICKER && resultCode == RESULT_OK && data != null) {
            Uri selectedFileUri = data.getData();
            String filePath = getFilePathFromUri(this, selectedFileUri);
            Log.i(TAG, "Selected file uri: " + selectedFileUri.toString() + " filePath: " + filePath);
            Toast.makeText(this, "Selected file: " + filePath, Toast.LENGTH_LONG).show();
            ArrayList<String> pathList = new ArrayList<>();
            pathList.add(filePath);

            mStreamView.getASPEngineDelegate().uploadFiles(pathList, "/sdcard/Download/");
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        DialogHelper.getInstance().reset();
        mStreamView.getASPEngineDelegate().unregisterFileTransferListener(mFileTransferListener);
        mStreamView.dispose();
        mStreamView = null;
    }

    public String getFilePathFromUri(Context context, Uri uri) {
        InputStream inputStream = null;
        try {
            inputStream = context.getContentResolver().openInputStream(uri);
            File tempFile = new File(context.getCacheDir(), "tempfile");
            OutputStream outputStream = new FileOutputStream(tempFile);

            byte[] buffer = new byte[4 * 1024];
            int read;
            while ((read = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, read);
            }

            outputStream.flush();
            inputStream.close();
            outputStream.close();
            return tempFile.getAbsolutePath();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return "";
    }
}