package com.example.aspdemo.ui.login;

import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import androidx.lifecycle.ViewModel;

import android.util.Patterns;

import com.example.aspdemo.ADHelper;
import com.example.aspdemo.data.LoginDataSource;
import com.example.aspdemo.data.LoginRepository;
import com.example.aspdemo.data.Result;
import com.example.aspdemo.data.model.LoggedInUser;
import com.example.aspdemo.R;

import java.util.regex.Pattern;

public class LoginViewModel extends ViewModel {

    private MutableLiveData<LoginFormState> loginFormState = new MutableLiveData<>();
    private MutableLiveData<LoginResult> loginResult = new MutableLiveData<>();
    private LoginRepository loginRepository;

    LoginViewModel(LoginRepository loginRepository) {
        this.loginRepository = loginRepository;
    }

    LiveData<LoginFormState> getLoginFormState() {
        return loginFormState;
    }

    LiveData<LoginResult> getLoginResult() {
        return loginResult;
    }

    public void login(String username, String password, LoginDataSource.ILoginCallback callback) {
        // can be launched in a separate asynchronous job
        loginRepository.login(username, password, new LoginDataSource.ILoginCallback() {
            @Override
            public void onLoginSuccess(Result<LoggedInUser> user) {
                LoggedInUser data = ((Result.Success<LoggedInUser>) user).getData();
                loginResult.setValue(new LoginResult(new LoggedInUserView(data.getDisplayName())));
            }

            @Override
            public void onLoginFailed(Result<LoggedInUser> user) {
                loginResult.setValue(new LoginResult(R.string.login_failed));
            }
        });
    }

    public void loginDataChanged(String username, String password, String regionNumber) {
        if (!isUserNameValid(username)) {
            loginFormState.setValue(new LoginFormState(R.string.invalid_username, null, null));
        } else if (!isPasswordValid(password)) {
            loginFormState.setValue(new LoginFormState(null, R.string.invalid_password, null));
        } else if (!isOrganizationIdValid(regionNumber)) {
            loginFormState.setValue(new LoginFormState(null, null, R.string.invalid_organization_id));
        } else {
            loginFormState.setValue(new LoginFormState(true));
        }
    }

    // A placeholder username validation check
    private boolean isUserNameValid(String username) {
        if (username == null) {
            return false;
        }
        if (username.contains("@")) {
            return Patterns.EMAIL_ADDRESS.matcher(username).matches();
        } else {
            return !username.trim().isEmpty();
        }
    }


    // A placeholder password validation check
    private boolean isPasswordValid(String password) {
        return password != null && password.trim().length() > 0;
    }

    // A placeholder regionNumber validation check
    private boolean isOrganizationIdValid(String id) {
        return id != null && id.trim().length() > 0/*
                && (ADHelper.ORGANIZATION_ID_PATTERN1.matcher(id).matches()
                    || ADHelper.ORGANIZATION_ID_PATTERN2.matcher(id).matches())*/;
    }
}